/*
 *  cOptFlow.cpp
 *  extractMotionHistogram
 *
 *  Created by Kris Kitani on 5/25/11. (kkitani@cs.cmu.edu)
 *  Copyright 2011 Kris Kitani. All rights reserved.
 *
 */

#include "cOptFlow.h"

void cOptFlow::init(int maxCount){
	m_maxCount = maxCount;
	flags = 0;
	quality = 0.01;
	min_distance = 10;
	avg_blocksize = 3;
	count = 0;
	pcount = 0;
	pts_cur = (CvPoint2D32f*)cvAlloc(m_maxCount*sizeof(CvPoint2D32f));
	pts_prv = (CvPoint2D32f*)cvAlloc(m_maxCount*sizeof(CvPoint2D32f));
	pts_cur_trk = (CvPoint2D32f*)cvAlloc(m_maxCount*sizeof(CvPoint2D32f));
	pts_prv_trk = (CvPoint2D32f*)cvAlloc(m_maxCount*sizeof(CvPoint2D32f));
	stat   = (char*)cvAlloc(m_maxCount);
	eig = 0;
}

int cOptFlow::trackPoints(IplImage *gry){
	
	if(!eig){
		pgry = cvCreateImage(cvGetSize(gry), 8,1);
		pyr  = cvCreateImage(cvGetSize(gry), 8,1);
		ppyr = cvCreateImage(cvGetSize(gry), 8,1);
		eig  = cvCreateImage(cvGetSize(gry),32,1);
		tmp  = cvCreateImage(cvGetSize(gry),32,1);
		t  = cvCreateImage(cvGetSize(gry),8,1);
	}
	
	count = m_maxCount;
	cvGoodFeaturesToTrack(gry,eig,tmp,pts_cur,&count,quality,min_distance,NULL,3,0,0.04);	
	
	for(i=0; i<count; i++ ){
		pts_cur_trk[i] = pts_cur[i]; // current
		pts_prv_trk[i] = pts_prv[i]; // previous
	}
	
	k=0;
	
	if(pcount>0 && count>0){
		cvCalcOpticalFlowPyrLK(pgry,gry,ppyr,pyr,pts_prv_trk,pts_cur_trk,count,cvSize(11,11),3,stat,0,
							   cvTermCriteria(CV_TERMCRIT_ITER+CV_TERMCRIT_EPS,20,0.03), flags );
		flags |= CV_LKFLOW_PYR_A_READY;
		for( i = k = 0; i < count; i++ )
			if( stat[i]==1 ){
				pts_cur_trk[k] = pts_cur_trk[i];
				pts_prv_trk[k] = pts_prv_trk[i];
				k++;
			}
	}
	
	pcount = count;
	count = k;
	
	cvCopy(gry, pgry);
	CV_SWAP( ppyr, pyr, stmp );
	CV_SWAP( pts_prv, pts_cur, spts); // keep all point
	
	return count;
}

void  cOptFlow::getCurrentTrackedPoints(CvPoint2D32f **ctrk){
	*ctrk = pts_cur_trk;
}

void  cOptFlow::getPreviousTrackedPoints(CvPoint2D32f **ptrk){
	*ptrk = pts_prv_trk;
}

void  cOptFlow::getCount(int *c){
	*c = count;
}

